<?php
namespace Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Teknisa\GER\Utilities\Environment;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\Controller;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO\Response\Notification;
use Teknisa\Libs\Util\NewCode;

class Ger03000ConclusaoPrepOrcaController extends \Teknisa\GER\Controllers\TeknisaCrud {
	protected $crudService;
	protected $connection;
	
	public function __construct($crudService, ParameterBag $parameterBag, $connection, $service, $newCode, $environment) {
		$this->crudService	= $crudService;
		$this->connection	= $connection;
		$this->service		= $service;
		$this->newCode		= $newCode;
		$this->environment	= $environment;
        parent::__construct($crudService, $parameterBag);
	}
	
	public function checkAcesso(DTO\Request $request,  DTO\Response $response) {
		
		$currentRow = $request->getDataSet()->getRows()[0];
		$stmt = $this->connection->prepare("
			SELECT O.CDOPERADOR, O.NMOPERADOR, F.CDFILIAL, F.IDACESSOAPROV
			  FROM FILIALOPEROP F, OPERADOR O
			 WHERE O.CDOPERADOR = :P_CDOPERADOR
			   AND O.CDOPERADOR = F.CDOPERADOR
			   AND F.CDFILIAL = :P_CDFILIAL"
		);
		$stmt->execute(
			array(
				'P_CDFILIAL' => $currentRow["CURRENTCDFILIAL"],
				'P_CDOPERADOR' => $currentRow["CDOPERADOR"]
			)
		);
		$result = $stmt->fetchAll();
		$response->addDataSet(new DataSet("RESULT", $result));
	}
	
	public function autenticacao(DTO\Request $request,  DTO\Response $response) {
		$dados = $request->getDataSet()->getRows()[0];
		
		if(!empty($dados)){
			$stmt = $this->connection->prepare("
				SELECT CDOPERADOR, CDSENHAOPERWEB
				  FROM OPERADOR
				 WHERE CDOPERADOR = :P_CDOPERADOR"
			);
			$stmt->execute(
				array(
					'P_CDOPERADOR' => $dados["USER"]
				)
			);
			
			$result = $stmt->fetchAll();
			
			if(!empty($result)){
				
				$senha = md5($dados["PASSWORD"]);
				
				if($senha == $result[0]["CDSENHAOPERWEB"]){
					
					$stmt1 = $this->connection->prepare("
					SELECT CDOPERADOR,	IDACESSOAPROV,   CDFILIAL   
					  FROM FILIALOPEROP
					 WHERE CDOPERADOR = :P_CDOPERADOR
					   AND CDFILIAL = :P_CDFILIAL"
				);
				$stmt1->execute(
					array(
						'P_CDOPERADOR' => $dados["USER"],
						'P_CDFILIAL' => $dados["CDFILIAL"]
					)
				);
				$result1 = $stmt1->fetchAll();
					if($result1[0]["IDACESSOAPROV"] == 'S'){
						$result[0]["AUTENTIC"] = true;
					}else{
						$result[0]["AUTENTIC"] = false;
					}
				}else{
					$result[0]["AUTENTIC"] = false;
				}
			}else{
					$result[0]["AUTENTIC"] = false;
			}
			
			$response->addDataSet(new DataSet("RESULT", $result));
		}
	}
	
	public function checkEtapas(DTO\Request $request,  DTO\Response $response) {
		$currentRow = $request->getDataSet()->getRows()[0];

		$stmt = $this->connection->prepare("
			SELECT COUNT(1) ETAPAS
	          FROM CALENPREORCAGOP CA
	         WHERE CA.NRANOORCAGOP = :P_NRANOORCAGOP
	           AND CA.IDPERIODOORCA = :P_IDPERIODOORCA
	           AND CA.NRPERIODOORCA = :P_NRPERIODOORCA
	           AND CA.NRCALENPREORCAGOP NOT IN (SELECT CA.NRCALENPREORCAGOP
	                                              FROM CALENPREORCAGOP CA, CONCPREORCAGOP CO
	                                             WHERE CA.NRCALENPREORCAGOP = CO.NRCALENPREORCAGOP
	                                               AND CA.NRANOORCAGOP = :P_NRANOORCAGOP
	                                               AND CA.IDPERIODOORCA = :P_IDPERIODOORCA
	                                               AND CA.NRPERIODOORCA = :P_NRPERIODOORCA
	                                               AND CO.IDSTATUS = 'CONCLUIDO'
	                                               AND CO.CDFILIAL = :P_CDFILIAL)"
		);
		try{
			$stmt->execute(
				array(
					'P_NRANOORCAGOP'  => $currentRow["ANO"],
					'P_IDPERIODOORCA' => $currentRow["IDPERIODO"],
					'P_NRPERIODOORCA' => $currentRow["NRPERIODO"],
					'P_CDFILIAL'	  => $currentRow["FILIAL"]
				)
			);
			$result = $stmt->fetchAll();
			$response->addDataSet(new DataSet("ETAPAS", $result));
			
        }catch(\Exception $ex){
        	$response->setError(new Error($ex->getMessage(), 500));
        }
	}
	
	public function checkOrcamento(DTO\Request $request,  DTO\Response $response){
		$currentRow = $request->getDataSet()->getRows()[0];
		$stmt = $this->connection->prepare("
			SELECT COUNT(1) ORCAMENTOS 
			  FROM ORCAMENTOGOP 
			 WHERE NRANOORCAGOP = :P_NRANOORCAGOP
			   AND NRPERIODOORCA = :P_NRPERIODOORCA
			   AND IDPERIODOORCA = :P_IDPERIODOORCA
			   AND CDFILIAL = :P_CDFILIAL"
		);
		try{
			$stmt->execute(
				array(
					'P_NRANOORCAGOP' => $currentRow["NRANOORCAGOP"],
					'P_NRPERIODOORCA' => $currentRow["NRPERIODOORCA"],
					'P_IDPERIODOORCA' => $currentRow["IDPERIODOORCA"],
					'P_CDFILIAL' => $currentRow["CDFILIAL"]
				)
			);
		
			
			$result = $stmt->fetchAll();
			$response->addDataSet(new DataSet("ORCAMENTOS", $result));
			
        }catch(\Exception $ex){
        	$response->setError(new Error($ex->getMessage(), 500));
        }
	}
	
	public function checkExistAprova(DTO\Request $request,  DTO\Response $response){
		$currentRow = $request->getDataSet()->getRows()[0];
		
		$stmt = $this->connection->prepare("
			SELECT  MAX(NRAPROVAORCAGOP) NRAPROVAORCAGOP
			   FROM APROVAORCAGOP 
			  WHERE NRORCAMENTOGOP = :P_NRORCAMENTOGOP"
		);
		try{
			$stmt->execute(
				array(
					'P_NRORCAMENTOGOP' => $currentRow["NRORCAMENTOGOP"]
				)
			);
		
			
			$result = $stmt->fetchAll();
			$response->addDataSet(new DataSet("EXISTE", $result));
			
        }catch(\Exception $ex){
        	$response->setError(new Error("Nao foi possivel processar os dados enviados   --- " . $ex->getMessage(), 500));
        }
	}

	
	public function getOrcamento(DTO\Request $request,  DTO\Response $response){
		$currentRow = $request->getDataSet()->getRows()[0];
		$stmt = $this->connection->prepare("
			SELECT NRORCAMENTOGOP 
			  FROM ORCAMENTOGOP 
			 WHERE NRANOORCAGOP = :P_NRANOORCAGOP
			   AND NRPERIODOORCA = :P_NRPERIODOORCA
			   AND IDPERIODOORCA = :P_IDPERIODOORCA
			   AND CDFILIAL = :P_CDFILIAL"
		);
		try{
			$stmt->execute(
				array(
					'P_NRANOORCAGOP' => $currentRow["NRANOORCAGOP"],
					'P_NRPERIODOORCA' => $currentRow["NRPERIODOORCA"],
					'P_IDPERIODOORCA' => $currentRow["IDPERIODOORCA"],
					'P_CDFILIAL' => $currentRow["CDFILIAL"]
				)
			);
			
			$result = $stmt->fetchAll();
			$response->addDataSet(new DataSet("ORCAMENTO", $result));
			
        }catch(\Exception $ex){
        	$response->setError(new Error("Nao foi possivel processar os dados enviados   --- " . $ex->getMessage(), 500));
        }
	}
	
	public function conclusaoPrep(DTO\Request\Filter $request, DTO\Response $response) {
		$conditions = $request->getFilterCriteria()->getConditions();
        $rows = $this->crudService->getFilteredDataSet("ger03000_concpreporcagop", $conditions)->getRows();
        $parents = array();

        foreach ($rows as &$row) {
            if (!isset($parents[$row['NRORCAMENTOGOP']])) {
                $parents[$row['NRORCAMENTOGOP']] = array(
                    'ID'        		=> $row['NRORCAMENTOGOP'],
                    'NMFILIAL'          => $row['NMFILIAL'],
                    'CDFILIAL'        	=> $row['CDFILIAL'],
                    'ITENS'     		=> null,
                    'IDPERIODOORCA'     => $row['IDPERIODOORCA'],
                    'NRANOORCAGOP'      => $row['NRANOORCAGOP'],
                    'DSETAPA'           => $row['DSETAPA'],
                    'DSOBSERVACAO'      => $row['DSOBSERVACAO'],
                    'DTINICIALPREP'     => $row['DTINICIALPREP'],
                    'DTFINALPREP'       => $row['DTFINALPREP'],
                    'IDSTATUS'          => $row['IDSTATUSGERAL'],
                    'NMGRUPGERENTEOP'   => $row['NMGRUPGERENTEOP'],
                    'NMOPERADOR'        => $row['NMOPERADOR'],
                    'NRPERIODOORCA'		=> $row['NRPERIODOORCA'],
                    'NRORCAMENTOGOP'    => $row['NRORCAMENTOGOP']
                );
            }
            $row['ANO'] 		= $row['NRANOORCAGOP'];
            $row['IDPERIODO']	= $row['IDPERIODOORCA'];
            $row['NRPERIODO']	= $row['NRPERIODOORCA'];
            $row['FILIAL']		= $row['CDFILIAL'];
            $row['DATAINICIAL']	= $row['DTINICIALPREP'];
            $row['DATAFINAL']	= $row['DTFINALPREP'];
     
            $row['NMFILIAL']		= null;
        	$row['CDFILIAL']		= null;
        	$row['DSETAPA']			= null;
        	$row['IDPERIODOORCA']	= null;
        	$row['NRANOORCAGOP']	= null;
        	$row['DSOBSERVACAO']	= null;
        	$row['DTINICIALPREP']	= null;
        	$row['DTFINALPREP'] 	= null;
        	
            $row['PARENT'] = $parents[$row['NRORCAMENTOGOP']]['ID']; 
        };
        $datasetPais   = array_values($parents);
        $datasetFilhos = array_values($rows);
        $dataset = array_merge($datasetPais, $datasetFilhos);
        
        $response->addDataSet(new DataSet("rows", $dataset));
	}
	
	public function updateBudget(DTO\Request $request,  DTO\Response $response) {
		$dataset = $request->getDataSet();
		$rows = $dataset->getRows();
		try{
			$this->service->beginTransaction();
	
			foreach($rows as $row) {
				if($row['NRCONCPREORCAGOP'] != null) {
					$params = array(
						'P_NRCONCPREORCAGOP' => strval($row['NRCONCPREORCAGOP'])
					);
					
					$this->service->updateBudget($params);	
				}else {
					$params = array(
						'P_NRCONCPREORCAGOP'	=> $this->newCode->getCodeByTableName("CONCPREORCAGOP", $this->environment->getNrorg()),
						'P_IDTIPOORCAOP'		=> $row['IDTIPOORCAOP'],
						'P_CDFILIAL'			=> $row['FILIAL'],
						'P_NRORG'				=> $this->environment->getNrorg(),
						'P_IDATIVO'				=> 'S',
						'P_NRORCAMENTOGOP'		=> $row['NRORCAMENTOGOP'],
						'P_DTINICIALPREP'		=> $row['DATAINICIAL'],
						'P_DTFINALPREP'			=> $row['DATAFINAL']
					);
	
					$this->service->insertBudget($params);
				}
			}
				$this->service->commit();
			}catch(\Exception $ex) {
				$this->service->rollback();
				$response->setError(new Error("Não foi possível processar os dados enviados" . $ex->getMessage(), 500));
			}
	}
}